/*******************************************************************************
*    BitmapButton für wxWidget 1.0.                                            *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <sys/time.h>
#include "BitmapButton.h"

/**
 * Konstruktor
 *
 * @param wxWindow *parent                Parentfenster
 * @param wxWindowID id                   ID des Objektes
 * @param wxBitmap bitmap                 Hintergrundbild des Knopfes
 * @param wxBitmap background             Bitmap, das UNTER dem Knopf steht, damit
 *                                        eine "Transparenz" geleistet werden kann
 * @param wxString label                  String anzuzeigen
 * @param wxPoint pos                     Position des Knopfes
 * @param wxSize size                     Größe des Knopfes
 * @param wxBitmap toggle                 Bitmap für den gedruckten Knopf, wenn
 *                                        der Knopf ein "togglebutton" ist.
 *                                        Ein Knopf ist "togglebutton", wenn dieses
 *                                        Parameter ein Bild ist. Will man kein ToggleButton
 *                                        erzeugen, wird hier wxImage() angegeben.
 * @param wxColour textColor              Farbe des Textes
 * @param bool toggledStatus              Anfangsstatus des ToggleButtons
 */
BitmapButton::BitmapButton(wxWindow *parent, wxWindowID id, const wxBitmap& bitmap, wxBitmap& background,
                   const wxString &label, const wxPoint& pos, const wxSize& size, const wxBitmap& toggle,
                   const wxColour& textColour, bool toggledStatus)
: wxControl(parent, id, pos, size, wxTRANSPARENT_WINDOW|wxBORDER_NONE)
{
  this->parent = parent;
  this->id = id;
  this->bitmap = bitmap;
  this->background = background;
  this->pos = pos;
  this->size = size;
  this->label = label;
  this->textColour = textColour;
  this->toggle = toggle;
  this->labelX = this->labelY = -1;
  this->fontSize = 9;
  this->bold = this->italic = false;
  this->startPressed = 0;
  this->startedEvent = false;
  this->longPressTime = 1000L;

  this->isToggle = false;
  if(toggle.GetHeight() != -1)
    this->isToggle = true;
  this->pressed = this->prevToggled = toggledStatus;
  Connect(wxEVT_LEFT_UP, wxCommandEventHandler(BitmapButton::OnClick), NULL, this);
  Connect(wxEVT_LEFT_DOWN, wxCommandEventHandler(BitmapButton::OnClick), NULL, this);
  Connect(wxEVT_PAINT, wxPaintEventHandler(BitmapButton::OnPaint), NULL, this);
  Connect(wxEVT_SIZE, wxSizeEventHandler(BitmapButton::OnSize), NULL, this);
}

BitmapButton::~BitmapButton()
{
  Disconnect(wxEVT_LEFT_UP, wxCommandEventHandler(BitmapButton::OnClick), NULL, this);
  Disconnect(wxEVT_LEFT_DOWN, wxCommandEventHandler(BitmapButton::OnClick), NULL, this);
  Disconnect(wxEVT_PAINT, wxPaintEventHandler(BitmapButton::OnPaint), NULL, this);
  Disconnect(wxEVT_SIZE, wxSizeEventHandler(BitmapButton::OnSize), NULL, this);
}

/**
 * Ändert die Größe des Fonts
 *
 * @param int size                        Die neue Fontgröße
 */
void BitmapButton::setFontSize(int size)
{
  this->fontSize = size;
}

/**
 * Die Label wird im Fettschrift angezeigt
 *
 * @param bool bold                       true wenn die Label im Fettschrift ist,
 *                                        false ansonsten
 */
void BitmapButton::setBold(bool bold)
{
  this->bold = bold;
}

/**
 * Die Label wird im Kursiv angezeigt
 *
 * @param bool bold                       true wenn die Label im Kursiv ist,
 *                                        false ansonsten
 */
void BitmapButton::setItalic(bool italic)
{
  this->italic = italic;
}

/**
 * "Druckt" ein ToggleButton
 *
 * @param bool toggled                    Das Status des ToggleButton (true gedruckt, false nicht)
 */
void BitmapButton::setToggled(bool toggled)
{
  if(!this->startedEvent) // Nur ein Update, wenn der Knopf nicht gerade ein Event hat
    this->pressed = this->prevToggled = toggled;
}

/**
 * Ändert das Bild des gedruckten ToggleButtons
 *
 * @param wxBitmap toggle                 Das neue Bild
 */
void BitmapButton::setToggleBitmap(wxBitmap toggle)
{
  this->toggle = toggle;
}

/**
 * Ändert die Textfarbe
 *
 * @param wxColour colour                 Die neue Textfarbe
 */
void BitmapButton::setTextColour(wxColour colour)
{
  this->textColour = colour;
}

/**
 * Ändert die Zeit, die nötig ist, um ein "LongPress"
 * zu erzeugen (standard: 1000 Millisekunden)
 *
 * @param long time                       Die Zeit in Millisekunden
 */
void BitmapButton::setLongPressTime(long time)
{
  this->longPressTime = time;
}

/**
 * Callback-Funktion, wenn der Knopf gedruckt wird
 */
void BitmapButton::OnClick(wxCommandEvent& event)
{
  struct timeval tv;
  long now;
  bool data = false;

  if(event.GetEventType() == wxEVT_LEFT_DOWN)
  {
    this->pressed = true;
    gettimeofday(&tv, NULL);
    this->startPressed = (tv.tv_sec) * 1000 + (tv.tv_usec) / 1000;
    this->startedEvent = true;
    Refresh();
  }
  if(event.GetEventType() == wxEVT_LEFT_UP)
  {
    this->startedEvent = false;
    if(this->pressed)
    {
      gettimeofday(&tv, NULL);
      now = (tv.tv_sec) * 1000 + (tv.tv_usec) / 1000;
      wxCommandEvent event(wxEVT_COMMAND_BUTTON_CLICKED, this->id);
      event.SetEventObject(this);
      if((now - this->startPressed) > this->longPressTime)
        data = true;
      event.SetClientData(&data);
      this->GetEventHandler()->ProcessEvent(event);
    }
    if(this->isToggle)
    {
      if(this->prevToggled)
        this->pressed = false;
      this->prevToggled = !this->prevToggled;
    }
    else
      this->pressed = false;
    Refresh();
  }
}

/**
 * Callback-Funktion, wenn die Größe des Knopfes
 * geändert wird
 */
void BitmapButton::OnSize(wxSizeEvent& event)
{
  Refresh();
}

/**
 * X-Position des Labels
 *
 * @param int x                           Die Position des Labels
 *                                        Bei -1 wird es horizontal zentriert
 */
void BitmapButton::setLabelX(int x)
{
  this->labelX = x;
}

/**
 * Y-Position des Labels
 *
 * @param int y                           Die Position des Labels
 *                                        Bei -1 wird es vertikal zentriert
 */
void BitmapButton::setLabelY(int y)
{
  this->labelY = y;
}

/**
 * Callback-Funktion um den Knopf zu malen.
 */
void BitmapButton::OnPaint(wxPaintEvent& event)
{
  int p, lx, ly, alignX, alignY;
  int style = wxFONTSTYLE_NORMAL, weight= wxFONTSTYLE_NORMAL;

  if(this->italic)
    style = wxFONTSTYLE_ITALIC;
  if(this->bold)
    weight = wxFONTWEIGHT_BOLD;
  wxFont font(this->fontSize, wxFONTFAMILY_DEFAULT, style,
            weight, false, wxT("Courier 10 Pitch"));

  wxPaintDC dc(this);

  dc.Clear();
  dc.SetFont(font);

  dc.DrawBitmap(this->background.GetSubBitmap(wxRect(this->pos, this->size)), 0, 0, false);

  p = (this->pressed ? 1 : 0);

// Hintergrundbild
  if(this->isToggle)
    dc.DrawBitmap(this->pressed ? this->toggle : this->bitmap, p, p, true);
  else
    dc.DrawBitmap(this->bitmap, p, p, true);

// Text
  alignX = wxALIGN_CENTER_HORIZONTAL;
  alignY = wxALIGN_CENTER_VERTICAL;
  if(this->labelX != -1)
  {
    lx = this->labelX;
    alignX = wxALIGN_LEFT;
  }
  else
    lx = p;
  if(this->labelY != -1)
  {
    ly = this->labelY;
    alignY = wxALIGN_TOP;
  }
  else
    ly = p;
  dc.SetTextForeground(this->textColour);
  dc.DrawLabel(this->label, wxRect(lx, ly, this->size.GetWidth(), this->size.GetHeight()), alignX | alignY);
}
